import unittest
import os
import tempfile


from oplus import CONF, IDF, simulate



@unittest.skip("temporarily for jenkins")
class ZoneWithShadingSimple1(unittest.TestCase):
    idf_path = os.path.join(CONF.eplus_base_dir_path, "ExampleFiles", "4ZoneWithShading_Simple_1.idf")
    epw_path = os.path.join(CONF.eplus_base_dir_path, "WeatherData", "USA_CO_Golden-NREL.724666_TMY3.epw")

    def test_err(self):
        with tempfile.TemporaryDirectory() as temp_dir_path:
            idf = IDF(self.idf_path)
            idf.add_object('''OutputControl:Table:Style,Comma,JtoKWH;''')
            idf.add_object('''Output:Table:SummaryReports,AllSummary;''')
            s = simulate(idf, self.epw_path, temp_dir_path)
            self.assertIsNotNone(s.err)  # todo: test results !!



import os
import io
import re

import pandas as pd
from pandas.util.testing import assert_index_equal

from oplus.configuration import CONF
from oplus.util import EPlusDt, get_start_dt, get_copyright_comment, get_string_buffer


class EPWError(Exception):
    pass


EPW_COLUMNS = ("year", "month", "day", "hour", "minute", "datasource",
               "drybulb", "dewpoint", "relhum", "atmos_pressure", "exthorrad",
               "extdirrad", "horirsky", "glohorrad", "dirnorrad", "difhorrad",
               "glohorillum", "dirnorillum", "difhorillum", "zenlum", "winddir",
               "windspd", "totskycvr", "opaqskycvr", "visibility", "ceiling_hgt", "presweathobs",
               "presweathcodes", "precip_wtr", "aerosol_opt_depth", "snowdepth", "days_last_snow",
               "albedo", "liq_precip_depth", "liq_precip_rate")

WEEK_DAYS = ("Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday", "Sunday")


class EPWHeader:
    LOCATION = 0
    DESIGN_CONDITIONS = 1
    TYPICAL_EXTREME_PERIODS = 2
    GROUND_TEMPERATURES = 3
    HOLIDAYS_DAYLIGHT_SAVINGS = 4
    COMMENTS_1 = 5
    COMMENTS_2 = 6
    DATA_PERIODS = 7

    @staticmethod
    def copyright_comment():
        return get_copyright_comment(multi_lines=False)

    def __init__(self, header_s):
        self._l2 = []
        for line_s in header_s.strip().split("\n"):
            self._l2.append([cell.strip() for cell in line_s.split(",")])
        # check
        if len(self._l2[self.DATA_PERIODS]) != 7:
            raise EPWError("'DATA PERIODS' row must have 7 cells.")
        if self._l2[self.DATA_PERIODS][0] != "DATA PERIODS":
            raise EPWError("Last line of header must be 'DATA PERIODS'.")
        if self._l2[self.DATA_PERIODS][1] != "1":
            raise NotImplementedError("Can only manage epws with one data period.")
        if self._l2[self.DATA_PERIODS][2] != "1":
            raise NotImplementedError("Can only manage hourly epws.")

    def to_str(self, add_copyright=True):
        l2 = []
        if add_copyright:
            for i, row in enumerate(self._l2):
                if add_copyright and (i == self.COMMENTS_1):
                    copyright_comment = self.copyright_comment()
                    row = row.copy()
                    if not copyright_comment in row[1]:
                        row[1] = "%s; %s" % (copyright_comment, row[1])
                l2.append(row)
        else:
            l2 = self._l2
        return "\n".join([",".join(row) for row in l2])

    @property
    def start_day_of_week(self):
        return self._l2[self.DATA_PERIODS][4].strip()

    @start_day_of_week.setter
    def start_day_of_week(self, value):
        # TODO: check given value is ok
        self._l2[self.DATA_PERIODS][4] = value

    @property
    def start(self):  # TODO: rename to start day
        """
        (month, day)
        """
        return self._l2[self.DATA_PERIODS][5].split(",").map(int)

    @start.setter
    def start(self, value):
        self._l2[self.DATA_PERIODS][5] = "%s/%s" % value

    @property
    def end(self):  # TODO: rename to end_day
        """
        (month, day)
        """
        return self._l2[self.DATA_PERIODS][6].split(",").map(int)

    @end.setter
    def end(self, value):
        self._l2[self.DATA_PERIODS][6] = "%s/%s" % value

    @property
    def freq(self):
        return {"1": "H"}[self._l2[self.DATA_PERIODS][2]]


class EPW:
    epw_header_cls = EPWHeader

    @classmethod
    def get_epw_or_path(cls, epw_or_path, encoding=None):
        if isinstance(epw_or_path, str):
            return cls(epw_or_path, encoding=encoding)
        elif isinstance(epw_or_path, cls):
            return epw_or_path
        raise EPWError("'epw_or_path' must be an EPW or path.  Given object: '%s', type: '%s'." %
                       (epw_or_path, type(epw_or_path)))

    def __init__(self, path_or_buffer, encoding=None, start=None):
        """
        if path_or_buffer is a path, it must end by .epw
        """
        self._encoding = CONF.encoding if encoding is None else encoding
        buffer, path = get_string_buffer(path_or_buffer, "epw", self._encoding)

        try:
            self._df, self._header = parse_epw(buffer)
        except Exception as e:
            raise EPWError("Error while parsing epw. First check that given file exists"
                           "(if not, given path will have been considered as an idf content.\n%s" % e)

        self._start_dt = None if start is None else get_start_dt(start)

    @property
    def header(self):
        return self._header

    @property
    def freq(self):
        return self._header.freq

    def to_str(self, add_copyright=True):
        # header
        _content = self._header.to_str(add_copyright=add_copyright)

        # data
        _f = io.StringIO()
        self._df.reset_index().to_csv(_f, header=False, index=False)
        _content += "\n" + _f.getvalue()

        return _content

    def save_as(self, file_or_path, add_copyright=True):
        is_path = isinstance(file_or_path, str)

        # write to f
        f = (open(file_or_path, "w", encoding=self._encoding)
             if is_path else file_or_path)
        f.write(self.to_str(add_copyright=add_copyright))

        if is_path:
            f.close()

    def set_start(self, start):
        self._start_dt = get_start_dt(start)

    def df(self, start=None, datetime_index=None):
        """
        order: will impose a freq, should not be used with reference climate (where years may differ by month)
        """
        # start_dt and datetime index
        if start is None:
            start_dt = self._start_dt
        else:
            start_dt = get_start_dt(start)

        if datetime_index is None:
            datetime_index = start_dt is not None
        if (datetime_index is True) and (start_dt is None):
            # we get first value as start_dt
            i0 = self._df.index[0]
            start_dt = EPlusDt(i0[1], i0[2], i0[3], i0[4]).datetime(i0[0])

        # data frame
        df = self._df.copy()

        if not datetime_index:  # return if multi-index
            return df

        # manage datetime df
        start_standard_dt = EPlusDt.from_datetime(start_dt).standard_dt

        def index_to_dt(i):
            eplusdt = EPlusDt(i[1], i[2], i[3], i[4])
            _year = start_dt.year + 1 if eplusdt.standard_dt <= start_standard_dt else start_dt.year
            return eplusdt.datetime(_year)

        df.index = df.index.map(index_to_dt)
        df.sort(inplace=True)
        df = df.reindex(pd.date_range(df.index[0], df.index[-1], freq=self.freq))

        return df

    def set_df(self, value):
        if not isinstance(value, pd.DataFrame):
            raise EPWError("df must be a DataFrame")
        assert_index_equal(value.index, self._df.index)
        assert_index_equal(value.columns, self._df.columns)


def parse_epw(file_like, encoding=None):
    header_s = ""

    # header
    for line_s in file_like:
        header_s += line_s
        if re.match("^DATA PERIODS", line_s) is not None:
            break
    header = EPWHeader(header_s)

    # data
    df = pd.read_csv(file_like, header=None, low_memory=False, encoding=encoding)
    df.columns = EPW_COLUMNS[:len(df.columns)]
    df.index = pd.MultiIndex.from_arrays([df[c] for c in df.columns[:5]])
    for c in df.columns[:5].tolist():
        del df[c]

    return df, header



from .version import version as __version__

from oplus.configuration import CONF
from oplus.eio import EIO
from oplus.epw import EPW
from oplus.idf import IDF
from oplus.mtd import MTD
from oplus.err import ERR
from oplus.summary_table import SummaryTable
from oplus.output_table import OutputTable
from oplus.simulation import Simulation, simulate
from oplus.standard_output import StandardOutputFile



import os
import pandas as pd

from . import CONF


class ERRError(Exception):
    pass


class ERR:
    WARNING = 'Warning'
    FATAL = 'Fatal'
    SEVERE = 'Severe'

    CATEGORIES = (WARNING, FATAL, SEVERE)

    def __init__(self, path, encoding=None):
        assert os.path.isfile(path), "No file at given path: '%s'." % path
        self.path = path
        self.encoding = CONF.encoding if encoding is None else encoding

        self.df = None
        self.info = {}
        self._parse()

        self.simulation_step_list = list(set(self.df.columns.levels[0]))

    @property
    def content(self):
        with open(self.path, encoding=self.encoding) as f:
            return f.read()

    def _parse(self):
        # TODO: Manage information with ahead '*************'
        # TODO: Manage 'error flag' :
        # TODO:  it corresponds to error type for each error_category lines_s.split('=')[0] --> MultiIndex

        # first step: warmup
        simulation_step = 'Warmup'
        max_nb = int(1e4)
        step_df = pd.DataFrame(columns=self.CATEGORIES, index=range(0, max_nb))
        category, index_nb = None, None
        with open(self.path, encoding=self.encoding) as f:
            for var in enumerate(f):
                # line_nb = var[0]
                line_s = var[1].rstrip('\n')

                # GET GENERIC INFORMATION
                if 'Program Version,EnergyPlus' in line_s:
                    self.info['EnergyPlus Simulation Version'] = line_s.split(',')[2].rstrip('Version ')
                    self.info['IDD_Version'] = line_s.split('IDD_Version ')[1]
                elif 'EnergyPlus Warmup Error Summary' in line_s:
                    self.info['EnergyPlus Warmup Error Summary'] = line_s.split('. ')[1]
                elif 'EnergyPlus Sizing Error Summary' in line_s:
                    self.info['EnergyPlus Sizing Error Summary'] = line_s.split('. ')[1]
                elif 'EnergyPlus Completed Successfully' in line_s:
                    self.info['EnergyPlus Completed Successfully'] = line_s.split('--')[1]

                # PARSE AND ..
                elif '************* Beginning' in line_s:
                    # SET OUTPUT DATAFRAME
                    if self.df is None:
                        iterables = [simulation_step, step_df.columns]
                        columns = pd.MultiIndex.from_product(iterables)
                        self.df = pd.DataFrame(index=range(0, max_nb), columns=columns)
                        self.df[simulation_step] = step_df
                    else:
                        iterables = [simulation_step, step_df.columns]
                        columns = pd.MultiIndex.from_product(iterables)
                        multi_step_df = pd.DataFrame(index=range(0, max_nb), columns=columns)
                        multi_step_df[simulation_step] = step_df
                        self.df = self.df.join(multi_step_df)
                    # start new simulation step
                    simulation_step = line_s.split('Beginning ')[1]
                    step_df = pd.DataFrame(columns=self.CATEGORIES, index=range(0, max_nb))
                elif '** Warning **' in line_s:
                    category = 'Warning'
                    # new line (index) until next
                    series = step_df[category].dropna()
                    if len(series.index) == 0:
                        index_nb = 0
                    else:
                        index_nb = series.index[-1] + 1
                    step_df[category].loc[index_nb] = line_s.split('** Warning **')[1]
                elif '** Fatal **' in line_s:
                    category = 'Fatal'
                    series = step_df[category].dropna()
                    if len(series.index) == 0:
                        index_nb = 0
                    else:
                        index_nb = series.index[-1] + 1
                    # new line (index) until next
                    step_df[category].loc[index_nb] = line_s.split('** Fatal **')[1]
                elif '** Severe **' in line_s:
                    category = 'Severe'
                    series = step_df[category].dropna()
                    if len(series.index) == 0:
                        index_nb = 0
                    else:
                        index_nb = series.index[-1] + 1
                    # new line (index) until next
                    step_df[category].loc[index_nb] = line_s.split('** Severe **')[1]

                elif '**   ~~~   **' in line_s:  # if we are here, we are sure category and index_nb have been defined
                    # information to add to error
                    step_df[category].loc[index_nb] += '\n' + line_s.split('**   ~~~   **')[1]

            self.info = pd.Series(self.info, index=self.info.keys())

    def get(self, simulation_step=None, error_category=None):
        """
        Return error report dataframe.

        If not argument are given --> return raw report
        If only on argument is specify --> return swap dataframe report

        Parameters
        ----------
        simulation_step, multicolumns report dataframe level 0
        error_cat, multicolumns report dataframe level 1

        Returns
        -------
        Specify Dataframe
        """
        if simulation_step is None and error_category is None:
            return self.df.dropna(axis='rows', how='all')

        if simulation_step is not None:
            if simulation_step not in self.simulation_step_list:
                raise ERRError("The simulation_step '%s' is not referred in the error file." % simulation_step)

            if error_category is not None:
                if error_category not in self.CATEGORIES:
                    raise ERRError("The error_cat '%s' is wrong." % error_category)
                iterables = [simulation_step, error_category]
                columns = pd.MultiIndex.from_product(iterables)
                series = self.df[simulation_step][error_category].dropna(axis='rows', how='all')

                df = pd.DataFrame(index=series.index, columns=columns)
                df[simulation_step] = series
                return df

            return self.df[simulation_step].dropna(axis='rows', how='all')

        if error_category is not None:
            if error_category not in self.CATEGORIES:
                raise ERRError("The error_category '%s' is wrong." % error_category)
            df = self.df.copy()
            df.columns = df.columns.swaplevel(0, 1)
            return df[error_category].dropna(axis='rows', how='all')



from setuptools import setup, find_packages
from setuptools.command.install import install
from setuptools.command.develop import develop
import os
import subprocess


with open(os.path.join("oplus", "version.py")) as f:
    version = f.read().split("=")[1].strip().strip("'").strip('"')


_cmd = "conda install -y --file conda-requirements.txt".split(" ")


class OInstall(install):
    def run(self):
        subprocess.call("conda install -y --file conda-requirements.txt".split(" "))
        install.run(self)


class ODevelop(develop):
    def run(self):
        subprocess.call(_cmd)
        develop.run(self)


setup(
    name='oplus',

    version=version,

    packages=find_packages(),

    author="Geoffroy d'Estaintot",

    author_email="geoffroy.destaintot@openergy.fr",

    description="A python package for working with Energy Plus",

    long_description=open('README.md').read(),  # long_description,

    install_requires=[
        'plotly>=1.9.6,<2.0.0',
        "nose-exclude>=0.4.1,<1.0.0"  # for tests
        ],

    url='https://github.com/Openergy/oplus',

    classifiers=[
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "Natural Language :: French",
        "Operating System :: POSIX :: Linux",
        "Programming Language :: Python :: 3.4",
    ],

    keywords=['data', 'simulation'],

    package_data={'oplus': ['*.txt']},

    include_package_data=True,

    cmdclass=dict(install=OInstall, develop=ODevelop)
)
